#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h> 
#include <sys/stat.h> 
#include "xxtea.h"

char *SIGN = "jiaile";
char *KEY = "6uJL0SF5CQQuPZoW";


void decryptFile(const char *infile) {
    FILE *fin, *fout;
    char *buf, *data;
    unsigned long size;
    int keylen, signlen, retlen;

    keylen = strlen(KEY);
    signlen = strlen(SIGN);

    fin = fopen(infile, "rb");
    if(fin == NULL) {
        perror("[x] Can't open the input file\n");
        return;
    }

    fseek(fin, 0L, SEEK_END);
    size = ftell(fin);
    rewind(fin);
    buf = (char*)malloc(size);
    fread(buf, size, 1, fin);
    fclose(fin);

    data = xxtea_decrypt(buf + signlen, size - signlen, KEY, keylen, &retlen);
    if(data == NULL) {
        printf("[x] Failed to decrypt: %s\n", infile);
        free(buf);
        return;
    }
    free(buf);

    fout = fopen(infile, "wb");
    if(fout == NULL) {
        perror("[x] Can't open the output file\n");
        free(data);
        return;
    }
    fwrite(data, retlen, 1, fout);
    fclose(fout);
    free(data);
}

void decryptDirectory(const char *directoryPath, const char *extensions[], int numExtensions) {
    DIR *dir;
    struct dirent *entry;
    char path[1024];
    struct stat statbuf;

    if ((dir = opendir(directoryPath)) == NULL) {
        perror("[x] Failed to open directory\n");
        return;
    }

    while ((entry = readdir(dir)) != NULL) {
        snprintf(path, sizeof(path), "%s/%s", directoryPath, entry->d_name);
        if (stat(path, &statbuf) == -1)
            continue;

        if (S_ISDIR(statbuf.st_mode)) {
            if (strcmp(entry->d_name, ".") != 0 && strcmp(entry->d_name, "..") != 0) {
                decryptDirectory(path, extensions, numExtensions);
            }
        } else if (S_ISREG(statbuf.st_mode)) {
            for (int i = 0; i < numExtensions; i++) {
                if (strstr(entry->d_name, extensions[i]) != NULL) {
                    decryptFile(path);
                    break;
                }
            }
        }
    }
    closedir(dir);
}

int main(int argc, char *argv[]) {
    if (argc < 2) {
        printf("usage: decrypt [file|directory]\n");
        return -1;
    }

    struct stat statbuf;
    const char *extensions[] = {".png", ".jpg"};
    int numExtensions = sizeof(extensions) / sizeof(extensions[0]);

    if (stat(argv[1], &statbuf) != -1) {
        if (S_ISDIR(statbuf.st_mode)) {
            decryptDirectory(argv[1], extensions, numExtensions);
        } else if (S_ISREG(statbuf.st_mode)) {
            for (int i = 0; i < numExtensions; i++) {
                if (strstr(argv[1], extensions[i]) != NULL) {
                    decryptFile(argv[1]);
                    break;
                }
            }
        }
    } else {
        printf("[x] Invalid file or directory\n");
    }

    return 0;
}